#ifndef VISITOR_H_
#define VISITOR_H_

//Abstract visitor class.
//Used for compiler passes to transform the 
//AST in various ways.
#include "ast_decl.h"

class Visitor {
public:
virtual ~Visitor() {}

//Visit methods have the following form:
//Node* visit(T* n);
//for some T <: Node.
//n is the node being visited.  The visit method can modify
//the node it's visiting normally.
//The return value is in case the visitor wants to replace
//the node it's visiting with a node of a different type
//(after either deleting n or putting it somewhere else to
//prevent the object from being lost).
//Many visitors don't need to use that feature, and can just
//return n.
//FixedVisitor handles the replacing of the nodes; if deriving
//from FixedVisitor, the visit methods can just return a new
//object and will automatically take the place of n in the AST.

//All the visit methods are generated by looking at
//all the nodes in the AST.
//An actual good use of a #include being replaced by
//the contents of a file.
#include "visitor_visit_methods.h"

};
 
#endif /*VISITOR_H_*/
